import v3io.dataplane.container


class Session(object):

    def __init__(self, context, transport, access_key):
        self._context = context
        self._transport = transport
        self._access_key = access_key

    def new_container(self, container_name):
        return v3io.dataplane.container.Container(self._context, self._access_key, self._transport, container_name)



import os
import requests

import v3io.dataplane.response


class Transport(object):

    def __init__(self, logger, endpoints=None, max_connections=4, timeout=None):
        self._logger = logger
        self._endpoints = self._get_endpoints(endpoints)
        self._next_connection_pool = 0
        self._timeout = timeout

        # create a tuple of connection pools
        self._connection_pools = self._create_connection_pools(self._endpoints, max_connections)

    def encode_and_send(self,
                        container_name,
                        access_key,
                        encoder,
                        encoder_args,
                        output=None):

        # get request params with the encoder
        method, path, headers, body = encoder(container_name, access_key, encoder_args)

        # call the encoder to get the response
        response = self._http_request(method, path, headers, body)

        # create a response
        return v3io.dataplane.response.Response(output,
                                                response.status_code,
                                                headers,
                                                response.text)

    def _http_request(self, method, path, headers=None, body=None):
        endpoint, connection_pool = self._get_next_connection_pool()

        self._logger.debug_with('Tx', method=method, path=path, headers=headers, body=body)

        response = connection_pool.request(method,
                                           endpoint + path,
                                           headers=headers,
                                           data=body,
                                           timeout=self._timeout,
                                           verify=False)

        self._logger.debug_with('Rx', status_code=response.status_code, headers=response.headers, body=response.text)

        return response

    def _get_endpoints(self, endpoints):
        if endpoints is None:
            env_endpoints = os.environ.get('V3IO_API')

            if env_endpoints is not None:
                endpoints = env_endpoints.split(',')
            else:
                raise RuntimeError('Endpoints must be passed to context or specified in V3IO_API')

        endpoints_with_scheme = []

        for endpoint in endpoints:
            if not endpoint.startswith('http://') and not endpoint.startswith('https://'):
                endpoints_with_scheme.append('http://' + endpoint)
            else:
                endpoints_with_scheme.append(endpoint)

        return endpoints_with_scheme

    def _create_connection_pools(self, endpoints, max_connections):
        connection_pools = []

        for endpoint in endpoints:
            connection_pools.append((endpoint, requests.Session()))

        return tuple(connection_pools)

    def _get_next_connection_pool(self):

        # TODO: multithreading safe way to get a connection pool. for now
        # support only one target
        return self._connection_pools[0]



import ujson
import xml.etree.ElementTree


class Response(object):

    def __init__(self, output, status_code, headers, body):
        self.status_code = status_code
        self.body = body
        self.headers = headers
        self.output = None

        if output and self.body:
            if self.headers.get('Content-Type') == 'application/json':
                self.output = output(ujson.loads(self.body))

            # since there's no content type, look for xml start
            elif self.body[0] == '<':
                self.output = output(xml.etree.ElementTree.fromstring(self.body))

    def raise_for_status(self, expected_statuses=None):
        if (expected_statuses is None and self.status_code >= 300) \
                or (expected_statuses and self.status_code not in expected_statuses):
            raise RuntimeError('Request failed with status {0}: {1}'.format(self.status_code, self.body))


class Responses(object):

    def __init__(self):
        self.responses = []
        self.success = True

    def add_response(self, response):
        self.responses.append(response)

        if response.status_code != 200:
            self.success = False

    def raise_for_status(self):
        if not self.success:
            raise RuntimeError('Failed to put items')
